//=========================================================================
// SW(2007) AER model
// all obsevable variables and 7 structural shocks
//=========================================================================

//=========================================================================
// ENDOGENOUS VARIABLES
var  
zcapf rkf kf pkf cf invef yf labf wf rrf mc zcap rk k pk 
c inve y lab pinf w r a b g qs kpf kp spinf sw ;    

//=========================================================================
// EXOGENOUS INNOVATIONS
varexo 
ea eb eg eqs em epinf ew;  
 
//=========================================================================
// DEEP AND STUCTURAL PARAMETERS
parameters 
curvw curvp cgy constebeta cmaw cmap calfa czcap cbeta csadjcost ctou 
csigma chabb ccs cinvs cfc cindw cprobw cindp cprobp csigl clandaw 
crdpi crpi crdy cry crr crhoa crhoas crhob crhog crhols crhoqs crhoms 
crhopinf crhow conster cg cgamma clandap cbetabar cr cpie crk 
cw cikbar cik clk cky ciy ccy crkky cwhlc cwly crhow crhopinf ;

//=========================================================================
// fixed parameters
ctou    = .025;    // depreciation rate
clandaw = 1.5;     // elasticity of substitution labor
cg      = 0.18;    // gov't consumption output share
curvp   = 10;      // good markets kimball aggragator 
curvw   = 10;      // labor markets kimball aggragator 

cgamma  = 1;       // trend, if =1 no trend
cbeta   = 0.99;    // time discount factor

//=========================================================================
// Deep Parameters values: 
// phip is one plus the share of fixed costs in production,

cfc         = 1.61;     // (PHI) in the table 1A (AER 2007),
csadjcost   = 5.00;     // (phi) in the table 1A (AER 2007), is the steady-state elasticity of the capital adjustment cost function

calfa       = 0.20;      // (alpha) capital share
chabb       = 0.61;      // (h) Habit in consumtion     
cprobw      = 0.63;      // (xi_w) wage stickiness
cprobp      = 0.65;      // (xi_p) price stickiness
cindw       = 0.59;      // (i_w) wage indexation
cindp       = 0.47;      // (i_p) price indexation
csigl       = 1.92;      // (sigma_l) inverse of frish elasticity
csigma      = 1.39;      // (sigma_c) elasticity of substitution
czcap       = 0.36;      // ( czcap=(1-psi)/psi ) in the table 1A (AER 2007), is a positive
                         // function of the elasticity of the capital utilization 
                         // adjustment cost function and normalized to be between 0 1.
crpi        = 2.03;      // (r_{\pi}) taylor rules response to inflation
crr         = 0.87;      // (rho} )  taylor rules smoothing
cry         = 0.08;      // (r_{y}) taylor rules response to output gap
crdy        = 0.12;      // (r_{Dy}) taylor rules response to oputput growth

//=========================================================================
// Autoregressive paramters:
crhoa       = 0.5;      // (rho_a) autoregressive technology
crhob       = 0.0;      // (rho_b) autoregressive preference
crhog       = 0.5;      // (rho_g) autoregressive gov't
crhoqs      = 0.5;      // (rho_I) autoregressive investment
crhow       = 0 ;
crhopinf    = 0 ;

//=========================================================================
// Composite paramters derived from steady state
clandap     =  cfc;
cbetabar    = cbeta*cgamma^(-csigma);
cr          = cpie/(cbeta*cgamma^(-csigma));
crk         = (cbeta^(-1))*(cgamma^csigma) - (1-ctou);
cw          = (calfa^calfa*(1-calfa)^(1-calfa)/(clandap*crk^calfa))^(1/(1-calfa));
cikbar      = (1-(1-ctou)/cgamma);
cik         = (1-(1-ctou)/cgamma)*cgamma;
clk         = ((1-calfa)/calfa)*(crk/cw);
cky         = cfc*(clk)^(calfa-1);
ciy         = cik*cky;
ccy         = 1-cg-cik*cky;
crkky       = crk*cky;
cwhlc       = (1/clandaw)*(1-calfa)/calfa*crk*cky/ccy;
cwly        = 1-crk*cky;


//=========================================================================
//=========================================================================
// SW MODEL LINEARIZED EQUATIONS
model(linear); 
//#usmodel_stst;

//=========================================================================
// flexible economy
0*(1-calfa)*a + 1*a =  calfa*rkf+(1-calfa)*(wf)  ;
zcapf =  (1/(czcap/(1-czcap)))* rkf  ;
rkf =  (wf)+labf-kf ;
kf =  kpf(-1)+zcapf ;
invef = (1/(1+cbetabar*cgamma))* (  invef(-1) + cbetabar*cgamma*invef(1)+(1/(cgamma^2*csadjcost))*pkf ) +qs ;
pkf = -rrf-0*b+(1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b +(crk/(crk+(1-ctou)))*rkf(1) +  ((1-ctou)/(crk+(1-ctou)))*pkf(1) ;
cf = (chabb/cgamma)/(1+chabb/cgamma)*cf(-1) + (1/(1+chabb/cgamma))*cf(+1) +((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(labf-labf(+1)) - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(rrf+0*b) + b ;
yf = ccy*cf+ciy*invef+g  +  crkky*zcapf ;
yf = cfc*( calfa*kf+(1-calfa)*labf +a );
wf = csigl*labf 	+(1/(1-chabb/cgamma))*cf - (chabb/cgamma)/(1-chabb/cgamma)*cf(-1) ;
kpf =  (1-cikbar)*kpf(-1)+(cikbar)*invef + (cikbar)*(cgamma^2*csadjcost)*qs ;

//=========================================================================
// sticky price - wage economy
mc   =  calfa*rk+(1-calfa)*(w) - 1*a - 0*(1-calfa)*a ;
zcap =  (1/(czcap/(1-czcap)))* rk ;
rk   = w + lab - k ;
k    = kp(-1)+zcap ;
kp   = (1-cikbar)*kp(-1)+cikbar*inve + cikbar*cgamma^2*csadjcost*qs ;
inve = (1/(1+cbetabar*cgamma))* (  inve(-1) + cbetabar*cgamma*inve(1) + 
       (1/(cgamma^2*csadjcost))*pk ) + qs ;
pk   = - r + pinf(1) - 0*b +
       (1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b + 
       (crk/(crk+(1-ctou)))*rk(1) +  ((1-ctou)/(crk+(1-ctou)))*pk(1) ;
c    = (chabb/cgamma)/(1+chabb/cgamma)*c(-1) + (1/(1+chabb/cgamma))*c(+1) +
       ((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(lab-lab(+1)) - 
       (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(r-pinf(+1) + 0*b) +b ;
y    = ccy*c + ciy*inve + g + 1*crkky*zcap ;
y    = cfc*( calfa*k + (1-calfa)*lab + a );
pinf = (1/(1+cbetabar*cgamma*cindp)) * ( cbetabar*cgamma*pinf(1) + cindp*pinf(-1) + 
       ((1-cprobp)*(1-cbetabar*cgamma*cprobp)/ cprobp)/((cfc-1)*curvp+1)*(mc)) 
        + spinf ;
w    = (1/(1+cbetabar*cgamma))*w(-1) + 
       (cbetabar*cgamma/(1+cbetabar*cgamma))*w(1) + 
       (cindw/(1+cbetabar*cgamma))*pinf(-1) - 
       (1+cbetabar*cgamma*cindw)/(1+cbetabar*cgamma)*pinf + 
       (cbetabar*cgamma)/(1+cbetabar*cgamma)*pinf(1) + 
       (1-cprobw)*(1-cbetabar*cgamma*cprobw)/((1+cbetabar*cgamma)*cprobw) * 
       (1/((clandaw-1)*curvw+1)) * 
       (csigl*lab + (1/(1-chabb/cgamma))*c - ((chabb/cgamma)/(1-chabb/cgamma))*c(-1) -w)
        + 1*sw ;
r     = crpi*(1-crr)*pinf + cry*(1-crr)*(y-yf) + crdy*(y - yf - y(-1)+ yf(-1)) + crr*r(-1) + em ;

//=========================================================================
//  autoregressive processes
a     = crhoa*a(-1)  + ea;
b     = crhob*b(-1) + eb;
g     = crhog*(g(-1)) + eg;
qs    = crhoqs*qs(-1) + eqs;
spinf = crhopinf*spinf(-1) + epinf;
sw    = crhow*sw(-1) + ew; 


end; 
//=========================================================================
//=========================================================================

shocks;
var ea;
stderr 0.4618;
var eb;
stderr 1.8513;
var eg;
stderr 0.6090;
var eqs;
stderr 0.6017;
var em;
stderr 0.2397;
var epinf;
stderr 0.14;
var ew;
stderr 0.2397;
end;

//=========================================================================

estimated_params;
// PARAM NAME, INITVAL, LB, UB, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF
/*
crr,        0.8258,0.5,0.975,   BETA_PDF,0.75,0.10;
crdy,       0.2239,0.001,0.5,   NORMAL_PDF,0.125,0.1;
crhob,      .50,.01,.9999,    BETA_PDF,0.5,0.20;
*/

crhoa,      .94,.01,.9999,    BETA_PDF,0.6,0.20;
crhog,      .97,.01,.9999,    BETA_PDF,0.6,0.20;
crhoqs,     .66,.01,.9999,    BETA_PDF,0.6,0.20;
cgy,        .50,0.01,2.0,     NORMAL_PDF,0.5,0.25;

// The autoregressive paramters of the price and wage
// markup are set to zero.

csigl,      1.81,0.25,10,       NORMAL_PDF,2,0.75;
csigma,     1.41,0.25,3,        NORMAL_PDF,1.50,0.5;
chabb,      0.74,0.4,0.99,      BETA_PDF,0.7,0.1;

cindw,      0.83,0.01,0.99,     BETA_PDF,0.5,0.15;
cindp,      0.51,0.01,0.99,     BETA_PDF,0.5,0.15;
cprobw,     0.69,0.3,0.95,      BETA_PDF,0.7,0.1;
cprobp,     0.61,0.5,0.95,      BETA_PDF,0.7,0.10;

cfc,        1.66,1.0,3,         NORMAL_PDF,1.5,0.25;
czcap,      0.29,0.01,1,        BETA_PDF,0.5,0.20;
csadjcost,  4.90,2,15,          NORMAL_PDF,5,1.5;
calfa,      0.18,0.01,1.0,      NORMAL_PDF,0.2,0.05;

crpi,       2.15,1.0,3,         NORMAL_PDF,2.0,0.5;

stderr ea,  0.41,0.005,3,      INV_GAMMA_PDF,0.1,2;
stderr eg,  0.63,0.005,3,      INV_GAMMA_PDF,0.1,2;
stderr eqs, 0.75,0.005,3,      INV_GAMMA_PDF,0.1,2;
stderr em,  0.23,0.005,3,      INV_GAMMA_PDF,0.1,2;


stderr eb,    0.034,0.005,3,    INV_GAMMA_PDF,0.05,Inf;
stderr ew,    0.012,0.005,3,    INV_GAMMA_PDF,0.05,Inf;
stderr epinf, 0.002,0.001,3,    INV_GAMMA_PDF,0.01,Inf;

end;

//=========================================================================

varobs y c inve w lab r pinf ;


estimation(datafile=data_D,mode_compute=4,prefilter=0,
           mh_replic=100000,mh_nblocks=2,mh_jscale=0.15,mh_drop=0.2,bayesian_irf);
